@@ -1,8 +1,12 @@
 # -*- coding: latin-1 -*-
 # Copyright (c) 2008-2009 Michael Howitz
 # See also LICENSE.txt
-# $Id: base.py 913 2009-01-02 12:47:38Z mac $
+# $Id: base.py 1081 2009-03-28 20:03:50Z mac $
 
+from icemac.addressbook.i18n import MessageFactory as _
+import icemac.addressbook.browser.interfaces
+import icemac.addressbook.interfaces
+import icemac.addressbook.utils
 import transaction
 import z3c.form.button
 import z3c.form.field
@@ -14,18 +18,13 @@ import zope.interface
 import zope.security
 import zope.security.interfaces
 import zope.traversing.api
-
-import icemac.addressbook.browser.interfaces
-import icemac.addressbook.interfaces
-import icemac.addressbook.utils
-
-from icemac.addressbook.i18n import MessageFactory as _
+import zope.traversing.browser.interfaces
 
 
 class display_title(object):
 
     zope.component.adapts(
-        zope.interface.Interface, 
+        zope.interface.Interface,
         icemac.addressbook.browser.interfaces.IAddressBookLayer)
     zope.interface.implements(icemac.addressbook.interfaces.ITitle)
 
@@ -42,6 +41,11 @@ def create(form, class_, data):
     return obj
 
 
+def all_(*constraints):
+    """All given button constraints must evaluate to true."""
+    return lambda form: all(constraint(form) for constraint in constraints)
+
+
 class BaseAddForm(z3c.formui.form.AddForm):
     """Simple base add form."""
 
@@ -63,15 +67,27 @@ class BaseAddForm(z3c.formui.form.AddForm):
             transaction.doom()
             raise z3c.form.interfaces.ActionExecutionError(e)
 
+    @z3c.form.button.buttonAndHandler(_('Add'), name='add')
+    def handleAdd(self, action):
+        # because we define a new action we have to duplicate the
+        # existing action because otherwise we'll lose it.
+        super(BaseAddForm, self).handleAdd(self, action)
+
+    @z3c.form.button.buttonAndHandler(_('Cancel'), name='cancel')
+    def handleCancel(self, action):
+        self.next_url = 'parent'
+        self._finishedAdd = True
+
     def nextURL(self):
         if self.next_url == 'object':
             context = self.context[self._name]
         elif self.next_url == 'parent':
             context = self.context
         else:
-            raise ValueError("Don't know how to handle next_url %r.") 
+            raise ValueError("Don't know how to handle next_url %r.")
         return zope.component.getMultiAdapter(
-            (context, self.request), name="absolute_url")()
+            (context, self.request),
+            zope.traversing.browser.interfaces.IAbsoluteURL)()
 
 
 class BaseEditForm(z3c.formui.form.EditForm):
@@ -90,7 +106,7 @@ class BaseEditForm(z3c.formui.form.EditForm):
             self.redirect_to_next_url()
         else:
             return super(BaseEditForm, self).render()
-            
+
     def redirect_to_next_url(self):
         if self.next_url == 'object':
             target = self.context
@@ -99,7 +115,8 @@ class BaseEditForm(z3c.formui.form.EditForm):
         else:
             raise ValueError('next_url %r unknown' % self.next_url)
         target_url = zope.component.getMultiAdapter(
-            (target, self.request), name="absolute_url")()
+            (target, self.request),
+            zope.traversing.browser.interfaces.IAbsoluteURL)()
         if self.next_view:
             target_url += '/%s' % self.next_view
         self.request.response.redirect(target_url)
@@ -118,7 +135,7 @@ class BaseEditFormWithCancel(BaseEditForm):
     @z3c.form.button.buttonAndHandler(_('Apply'), name='apply')
     def handleApply(self, action):
         # because we define a new action we have to duplicate the
-        # existing action because otherwise we'll loose it.
+        # existing action because otherwise we'll lose it.
         super(BaseEditFormWithCancel, self).handleApply(self, action)
 
     @z3c.form.button.buttonAndHandler(_('Cancel'), name='cancel')
@@ -180,7 +197,7 @@ def can_access(uri_part):
             view = traverser.traverseRelativeURL(
                 form.request, form.context, uri_part)
         except (zope.security.interfaces.Unauthorized,
-                zope.security.interfaces.Forbidden, 
+                zope.security.interfaces.Forbidden,
                 LookupError):
             return False
         else:
