@@ -13,7 +13,7 @@
 ##############################################################################
 """Form Implementation
 
-$Id: form.py 93230 2008-11-21 09:14:51Z adamg $
+$Id: form.py 97381 2009-02-28 12:26:20Z nadako $
 """
 __docformat__ = "reStructuredText"
 import sys
@@ -21,7 +21,7 @@ import zope.interface
 import zope.component
 import zope.event
 import zope.lifecycleevent
-from z3c.pt.compat import ViewPageTemplateFile
+from z3c import ptcompat
 from zope.publisher import browser
 from zope.pagetemplate.interfaces import IPageTemplate
 from zope.schema.fieldproperty import FieldProperty
@@ -38,7 +38,7 @@ def applyChanges(form, content, data):
             continue
         # If the value is NOT_CHANGED, ignore it, since the widget/converter
         # sent a strong message not to do so.
-        if data[name] == interfaces.NOT_CHANGED:
+        if data[name] is interfaces.NOT_CHANGED:
             continue
         # Get the datamanager and get the original value
         dm = zope.component.getMultiAdapter(
@@ -106,6 +106,7 @@ class BaseForm(browser.BrowserPage):
     fields = field.Fields()
 
     label = None
+    labelRequired = _('<span class="required">*</span>&ndash; required')
     prefix = 'form.'
     status = ''
     template = None
@@ -130,9 +131,16 @@ class BaseForm(browser.BrowserPage):
         self.widgets.ignoreReadonly = self.ignoreReadonly
         self.widgets.update()
 
+    @property
+    def requiredInfo(self):
+        if self.labelRequired is not None and self.widgets is not None \
+            and self.widgets.hasRequiredFields:
+            return zope.i18n.translate(self.labelRequired, context=self.request)
+
     def extractData(self, setErrors=True):
         '''See interfaces.IForm'''
-        return self.widgets.extract(setErrors=setErrors)
+        self.widgets.setErrors = setErrors
+        return self.widgets.extract()
 
     def update(self):
         '''See interfaces.IForm'''
@@ -159,7 +167,8 @@ class DisplayForm(BaseForm):
 class Form(BaseForm):
     """The Form."""
     zope.interface.implements(
-        interfaces.IInputForm, interfaces.IButtonForm, interfaces.IHandlerForm)
+        interfaces.IInputForm, interfaces.IButtonForm,
+        interfaces.IHandlerForm, interfaces.IActionForm)
 
     buttons = button.Buttons()
 
@@ -168,6 +177,12 @@ class Form(BaseForm):
     acceptCharset = FieldProperty(interfaces.IInputForm['acceptCharset'])
     accept = FieldProperty(interfaces.IInputForm['accept'])
 
+    actions = FieldProperty(interfaces.IActionForm['actions'])
+    refreshActions = FieldProperty(interfaces.IActionForm['refreshActions'])
+
+    # common string for use in validation status messages
+    formErrorsMessage = _('There were some errors.')
+
     @property
     def action(self):
         """See interfaces.IInputForm"""
@@ -189,6 +204,8 @@ class Form(BaseForm):
         super(Form, self).update()
         self.updateActions()
         self.actions.execute()
+        if self.refreshActions:
+            self.updateActions()
 
     def __call__(self):
         self.update()
@@ -203,7 +220,6 @@ class AddForm(Form):
     ignoreReadonly = True
 
     _finishedAdd = False
-    formErrorsMessage = _('There were some errors.')
 
     @button.buttonAndHandler(_('Add'), name='add')
     def handleAdd(self, action):
@@ -242,7 +258,6 @@ class EditForm(Form):
     """A simple edit form with an apply button."""
     zope.interface.implements(interfaces.IEditForm)
 
-    formErrorsMessage = _('There were some errors.')
     successMessage = _('Data successfully updated.')
     noChangesMessage = _('No changes were applied.')
 
@@ -279,7 +294,7 @@ class FormTemplateFactory(object):
 
     def __init__(self, filename, contentType='text/html', form=None,
         request=None):
-        self.template = ViewPageTemplateFile(filename, content_type=contentType)
+        self.template = ptcompat.ViewPageTemplateFile(filename, content_type=contentType)
         zope.component.adapter(
             util.getSpecification(form),
             util.getSpecification(request))(self)
