@@ -13,7 +13,7 @@
 ##############################################################################
 """Form and Widget Framework Interfaces
 
-$Id: interfaces.py 93891 2008-12-11 09:11:57Z icemac $
+$Id: interfaces.py 96972 2009-02-22 03:01:46Z rogerineichen $
 """
 __docformat__ = "reStructuredText"
 
@@ -29,15 +29,17 @@ INPUT_MODE = 'input'
 DISPLAY_MODE = 'display'
 HIDDEN_MODE = 'hidden'
 
-class NOT_CHANGED(str):
+class NOT_CHANGED(object):
     def __repr__(self):
         return '<NOT_CHANGED>'
-NOT_CHANGED = NOT_CHANGED()
+NOT_CHANGED = NOT_CHANGED() 
 
-class NOVALUE(object):
+class NO_VALUE(object):
     def __repr__(self):
-        return '<NOVALUE>'
-NOVALUE = NOVALUE()
+        return '<NO_VALUE>'
+NO_VALUE = NO_VALUE()
+
+NOVALUE = NO_VALUE # BBB: the object was renamed to follow common naming style
 
 # ----[ Layer Declaration ]--------------------------------------------------
 
@@ -237,7 +239,7 @@ class IDataManager(zope.interface.Interface):
         If no value can be found, raise an error
         """
 
-    def query(default=NOVALUE):
+    def query(default=NO_VALUE):
         """Get the value.
 
         If no value can be found, return the default value.
@@ -418,7 +420,15 @@ class IWidget(ILocation):
         default=False,
         required=False)
 
-    def extract(default=NOVALUE, setErrors=True):
+    #ugly thing to remove setErrors parameter from extract
+    setErrors = zope.schema.Bool(
+        title=_('Set errors'),
+        description=_('A flag, when set, the widget sets error messages '
+                      'on calling extract().'),
+        default=True,
+        required=False)
+
+    def extract(default=NO_VALUE):
         """Extract the string value(s) of the widget from the form.
 
         The return value may be any Python construct, but is typically a
@@ -432,8 +442,6 @@ class IWidget(ILocation):
         not want to crash the system during an inproper request.
 
         If there is no value to extract, the default is to be returned.
-
-        setErrors: needs to be passed on to possible sub-widgets
         """
 
     def update():
@@ -460,7 +468,7 @@ class ISequenceWidget(IWidget):
     """
 
     noValueToken = zope.schema.ASCIILine(
-        title=_('NOVALUE Token'),
+        title=_('NO_VALUE Token'),
         description=_('The token to be used, if no value has been selected.'))
 
     terms = zope.schema.Object(
@@ -595,14 +603,26 @@ class IWidgets(IManager):
         default=False,
         required=True)
 
+    hasRequiredFields = zope.schema.Bool(
+        title=_('Has required fields'),
+        description=_('A flag set when at least one field is marked as '
+                      'required'),
+        default=False,
+        required=False)
+
+    #ugly thing to remove setErrors parameter from extract
+    setErrors = zope.schema.Bool(
+        title=_('Set errors'),
+        description=_('A flag, when set, the contained widgets set error '
+                      'messages on calling extract().'),
+        default=True,
+        required=False)
+
     def update():
         """Setup widgets."""
 
-    def extract(setErrors=True):
+    def extract():
         """Extract the values from the widgets and validate them.
-
-        setErrors: decides whether to set errors on self and on the widgets
-                   also needs to be passed on to sub-widgets
         """
 
 
@@ -773,6 +793,23 @@ class IHandlerForm(zope.interface.Interface):
         required=True)
 
 
+class IActionForm(zope.interface.Interface):
+    """A form that stores executable actions"""
+    
+    actions = zope.schema.Object(
+        title=_('Actions'),
+        description=_('A list of actons defined on the form'),
+        schema=IActions,
+        required=True)
+
+    refreshActions = zope.schema.Bool(
+        title=_('Refresh actions'),
+        description=_('A flag, when set, causes form actions to be '
+                      'updated again after their execution.'),
+        default=False,
+        required=True)
+
+
 class IContextAware(zope.interface.Interface):
     """Offers a context attribute.
 
@@ -841,6 +878,12 @@ class IForm(zope.interface.Interface):
                       'used in the UI.'),
         required=False)
 
+    labelRequired = zope.schema.TextLine(
+        title=_('Label required'),
+        description=_('A human readable text describing the form that can be '
+                      'used in the UI for rendering a required info legend.'),
+        required=False)
+
     prefix = zope.schema.BytesLine(
         title=_('Prefix'),
         description=_('The prefix of the form used to uniquely identify it.'),
