@@ -13,14 +13,10 @@
 ##############################################################################
 """Common z3c.form test setups
 
-$Id: testing.py 93891 2008-12-11 09:11:57Z icemac $
+$Id: testing.py 96521 2009-02-14 10:38:14Z adamg $
 """
 __docformat__ = 'restructuredtext'
-import binascii
-import doctest
 import os
-import re
-import xml.parsers.expat
 
 import zope.component
 import zope.interface
@@ -41,10 +37,10 @@ from z3c.form import form, interfaces, term, validator, widget
 from z3c.form.browser import radio, select, text, textarea
 from z3c.form.browser import file as fileWidget
 
-from z3c.pt.compat.testing import render
-from z3c.pt.compat.testing import OutputChecker
+from z3c.ptcompat.testing import render
+from z3c.ptcompat.testing import OutputChecker
 
-import z3c.pt.compat
+import z3c.ptcompat
 
 import lxml.html
 import lxml.doctestcompare
@@ -114,7 +110,8 @@ class IMySubObject(zope.interface.Interface):
     foofield = zope.schema.Int(
         title=u"My foo field",
         default=1111,
-        max=9999)
+        max=9999,
+        required=True)
     barfield = zope.schema.Int(
         title=u"My dear bar",
         default=2222,
@@ -160,6 +157,42 @@ class MyComplexObject(object):
         self.subobject=subobject
         self.name=name
 
+class IMySubObjectMulti(zope.interface.Interface):
+    foofield = zope.schema.Int(
+        title=u"My foo field",
+        default=None, #default is None here!
+        max=9999,
+        required=True)
+    barfield = zope.schema.Int(
+        title=u"My dear bar",
+        default=2222,
+        required=False)
+
+class MySubObjectMulti(object):
+    zope.interface.implements(IMySubObjectMulti)
+
+    foofield = FieldProperty(IMySubObjectMulti['foofield'])
+    barfield = FieldProperty(IMySubObjectMulti['barfield'])
+
+class IMyMultiObject(zope.interface.Interface):
+    listOfObject = zope.schema.List(
+        title = u"My list field",
+        value_type = zope.schema.Object(
+            title=u'my object widget',
+            schema=IMySubObjectMulti),
+    )
+    name = zope.schema.TextLine(title=u'name')
+
+class MyMultiObject(object):
+    zope.interface.implements(IMyMultiObject)
+
+    listOfObject = FieldProperty(IMyMultiObject['listOfObject'])
+    name = FieldProperty(IMyMultiObject['name'])
+
+    def __init__(self, name=u'', listOfObject=None):
+        self.listOfObject = listOfObject
+        self.name = name
+
 #
 #
 #############################
@@ -168,11 +201,11 @@ def setUp(test):
     test.globs = {'root': setup.placefulSetUp(True)}
 
 def setUpZPT(suite):
-    z3c.pt.compat.config.disable()
+    z3c.ptcompat.config.disable()
     setUp(suite)
 
 def setUpZ3CPT(suite):
-    z3c.pt.compat.config.enable()
+    z3c.ptcompat.config.enable()
     setUp(suite)
     zope.configuration.xmlconfig.XMLConfig('configure.zcml', z3c.pt)()
 
@@ -263,6 +296,7 @@ def setupFormDefaults():
         IPageTemplate, name=interfaces.DISPLAY_MODE)
 
     # Select Widget
+    zope.component.provideAdapter(select.ChoiceWidgetDispatcher)
     zope.component.provideAdapter(select.SelectFieldWidget)
     zope.component.provideAdapter(
         widget.WidgetTemplateFactory(getPath('select_input.pt'), 'text/html'),
@@ -310,10 +344,10 @@ def setupFormDefaults():
     zope.component.provideAdapter(converter.TimedeltaDataConverter)
     # Adapter for providing terms to radio list and other widgets
     zope.component.provideAdapter(term.BoolTerms)
-    zope.component.provideAdapter(term.choice_terms_multiplexer)
+    zope.component.provideAdapter(term.ChoiceTerms)
     zope.component.provideAdapter(term.ChoiceTermsVocabulary)
     zope.component.provideAdapter(term.ChoiceTermsSource)
-    zope.component.provideAdapter(term.collection_terms_multiplexer)
+    zope.component.provideAdapter(term.CollectionTerms)
     zope.component.provideAdapter(term.CollectionTermsVocabulary)
     zope.component.provideAdapter(term.CollectionTermsSource)
     # Adapter to create an action from a button
